package org.isakiev.fileManager;

import javax.swing.*;

import java.awt.event.*;
import java.awt.Dimension;
import java.io.File;
import java.io.IOException;
import java.util.logging.*;
import java.util.jar.*;

import org.isakiev.fileManager.contentManagers.*;
import org.isakiev.fileManager.contentModel.*;
import org.isakiev.fileManager.contentViewers.*;
import org.isakiev.fileManager.contentViewers.explorer.ExplorerStyleContentViewer;
import org.isakiev.fileManager.contentViewers.nc.NCStyleContentViewer;
import org.isakiev.fileManager.entities.*;
import org.isakiev.fileManager.utils.*;

/**
 * Main class.
 * @author Isakiev Ruslan
 */
public class Main extends JFrame{
	
	private IContentModel myModel;
	private IContentViewer myExplorerStyleViewer;
	private IContentViewer myNCStyleViewer;
	
	private SupportedViewerStyles myCurrentStyle;
	private JPanel myMainPanel;
	private ActionMap myActionMap;
	
	private static String VERSION = "File manager v1.6";
	private static String BASE_DIR = System.getProperty("user.home") ;
	private static String LOG_FILE_NAME = BASE_DIR + File.separator + "FMLog.html";
	private static String STATE_FILE_NAME = BASE_DIR + File.separator + "FMState.xml";
	private static String DEFAULT_JAR_NAME = "FileManager.jar";
	
	//used to change selection in menu bar when viewer is switched automatically
	private JRadioButtonMenuItem mySwitchToExplorerStyleButton;
	private JRadioButtonMenuItem mySwitchToNCStyleButton;
	private ButtonGroup myViewerStyleButtonGroup;
	
	private Logger myLogger;
	private static boolean logging = false;
	

	/**
	 * Creates file manager's frame.
	 */
	public Main() {
		
		//Create main frame
		super(VERSION);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		addComponentListener(myResizeListener);
		
		//Define jar file name
		defineMyJarName();
		
		//Initialize logger
		myLogger = Logger.getLogger("org.isakiev.fileManager");
		myLogger.setUseParentHandlers(false);
		if (logging) {
			myLogger.setLevel(Level.ALL);
			try {
				FileHandler htmlLogHandler = new FileHandler(LOG_FILE_NAME);
				htmlLogHandler.setFormatter(new HTMLFormatter(VERSION));
				myLogger.addHandler(htmlLogHandler);
				
			} catch (IOException e) {};
		} else {
			myLogger.setLevel(Level.OFF);
		}

		//UIManager.installLookAndFeel("NC L&F","org.isakiev.fileManager.contentViewers.NCStyleLookAndFeel");
		
		//Create menu bar
		setJMenuBar(createMenuBar());
		
		//Create viewer
		myModel = new ContentModel(new ContentManager());
		myExplorerStyleViewer = new ExplorerStyleContentViewer(myModel);
		myNCStyleViewer = new NCStyleContentViewer(myModel);

		myMainPanel = myNCStyleViewer.getMainPanel();
		myCurrentStyle = SupportedViewerStyles.NC_STYLE;
		getContentPane().add(myMainPanel);
		
		//Create action map
		myActionMap = myMainPanel.getActionMap();
		myActionMap.put(Actions.EXIT, myExitAction);
		myActionMap.put(Actions.SWITCH_TO_EXPLORER_STYLE, mySwitchToExplorerStyleAction);
		myActionMap.put(Actions.SWITCH_TO_NC_STYLE, mySwitchToNCStyleAction);
		myActionMap.put(Actions.SWITCH_NEXT_STYLE, mySwitchNextStyleAction);
		myActionMap.put(Actions.SHOW_FEATURES, myShowFeaturesAction);
		myActionMap.put(Actions.SHOW_HOT_KEYS, myShowFeaturesAction);
		myActionMap.put(Actions.SHOW_ABOUT, myShowFeaturesAction);

		
		//Create input map
		InputMap inputMap = myMainPanel.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_X,KeyEvent.ALT_MASK), Actions.SWITCH_NEXT_STYLE);
		
		//Add window listener
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				myExitAction.actionPerformed(null);
			}
		});

		//Load saved state
		loadState(STATE_FILE_NAME);
		
		//Show frame
		setVisible(true);
		
		//Show info
		if ( !(new File(STATE_FILE_NAME)).exists() ) {
			myShowFeaturesAction.actionPerformed(null);
		}
		

		myLogger.info("File manager started.");
	}
	
	private enum Actions {
		EXIT,
		SWITCH_TO_EXPLORER_STYLE,
		SWITCH_TO_NC_STYLE,
		SWITCH_NEXT_STYLE,
		SHOW_FEATURES,
		SHOW_HOT_KEYS,
		SHOW_ABOUT
	}

	
	/**
	 * Saves current file manager's state.
	 * @param fileName file name
	 */
	private void saveState(String fileName) {
		StateLoader sm = StateLoader.getInstance();

		//add frame state info
		sm.setSize(this.getSize());
		sm.setLocation(this.getLocation());
		sm.setStyle(myCurrentStyle);
		
		//add NC style viewer info
		ViewerState state = myNCStyleViewer.getState();
		String leftEntity = (state.getLeftEntity() != null ? state.getLeftEntity().getFile().getAbsolutePath() : null);
		String rightEntity = (state.getRightEntity() != null ? state.getRightEntity().getFile().getAbsolutePath() : null);
		String leftDirectory = (state.getLeftDirectory() != null ? state.getLeftDirectory().getFile().getAbsolutePath() : null);
		String rightDirectory = (state.getRightDirectory() != null ? state.getRightDirectory().getFile().getAbsolutePath() : null);
		sm.setLeftEntityString(leftEntity);
		sm.setRightEntityString(rightEntity);
		sm.setLeftDirectoryString(leftDirectory);
		sm.setRightDirectoryString(rightDirectory);
		sm.setActivePanelIndex(state.getActivePanelIndex());
			
		//add explorer style viewer info
		state = myExplorerStyleViewer.getState();
		String currentEntity = (state.getCurrentEntity() != null ? state.getCurrentEntity().getFile().getAbsolutePath() : null);
		String currentDirectory = (state.getCurrentDirectory() != null ? state.getCurrentDirectory().getFile().getAbsolutePath() : null);
		sm.setCurrentEntityString(currentEntity);
		sm.setCurrentDirectoryString(currentDirectory);
		sm.setDividerLocation(state.getDividerLocation());

		//write document
		sm.saveState(fileName);
	}
	
	/**
	 * Load current filemanager's state. 
	 * @param fileName file name
	 */
	public void loadState(String fileName) {
		
		//load document
		StateLoader sm = StateLoader.getInstance();
		sm.loadState(fileName);
				
		//apply frame state info
		this.setSize(sm.getSize());
		this.setLocation(sm.getLocation());
		
		if (sm.getStyle() == SupportedViewerStyles.EXPLORER_STYLE) {
			mySwitchToExplorerStyleAction.actionPerformed(null);
		} else {
			mySwitchToNCStyleAction.actionPerformed(null);
		}
		
		//apply NC style viewer state info
		IEntity userDirectory = myModel.getEntity(BASE_DIR);
		IEntity leftEntity = (sm.getLeftEntityString() == null ? null : myModel.getEntity(sm.getLeftEntityString()));
		IEntity rightEntity = (sm.getRightEntityString() == null ? null : myModel.getEntity(sm.getRightEntityString()));
		IEntity leftDirectory = myModel.getEntity(sm.getLeftDirectoryString());
		if (leftDirectory == null) {
			leftDirectory = userDirectory;
		}
		IEntity rightDirectory = myModel.getEntity(sm.getRightDirectoryString());
		if (rightDirectory == null) {
			rightDirectory = userDirectory;
		}
		
		int activePanelIndex = sm.getActivePanelIndex();

		ViewerState state = new ViewerState(SupportedViewerStyles.NC_STYLE);
		state.setNCState(leftDirectory, leftEntity, rightDirectory, rightEntity, activePanelIndex);
		myNCStyleViewer.setState(state);

		//apply Explorer style viewer state info
		IEntity currentEntity = myModel.getEntity(sm.getCurrentEntityString());
		IEntity currentDirectory = myModel.getEntity(sm.getCurrentDirectoryString());
		int dividerLocation = sm.getDividerLocation();
		state = new ViewerState(SupportedViewerStyles.EXPLORER_STYLE);
		state.setExplorerState(currentDirectory, currentEntity, dividerLocation);
		myExplorerStyleViewer.setState(state);
	}
	
	
	/**
	 * Creates main menu bar.
	 * @return main menu bar
	 */
	private JMenuBar createMenuBar() {
		
		JMenuBar menuBar = new JMenuBar();

		//File
		JMenu fileMenu = new JMenu("File");
		fileMenu.add(myExitAction);
		
		//View
		JMenu viewMenu = new JMenu("View");

		myViewerStyleButtonGroup = new ButtonGroup();
		mySwitchToExplorerStyleButton = new JRadioButtonMenuItem(mySwitchToExplorerStyleAction);
		mySwitchToNCStyleButton = new JRadioButtonMenuItem(mySwitchToNCStyleAction);
		myViewerStyleButtonGroup.add(mySwitchToExplorerStyleButton);
		myViewerStyleButtonGroup.add(mySwitchToNCStyleButton);
		myViewerStyleButtonGroup.setSelected(mySwitchToNCStyleButton.getModel(), true);
		
		viewMenu.add(mySwitchToExplorerStyleButton);
		viewMenu.add(mySwitchToNCStyleButton);
		viewMenu.addSeparator();
		JMenu lookAndFeelMenu = createLookAndFeelMenu(this);
		viewMenu.add(lookAndFeelMenu);
		
		//About
		JMenu aboutMenu = new JMenu("About");
		aboutMenu.add(myShowFeaturesAction);
		aboutMenu.add(myShowHotKeysAction);
		aboutMenu.add(myShowAboutAction);
		
		//Add top level menues
		menuBar.add(fileMenu);
		menuBar.add(viewMenu);
		menuBar.add(aboutMenu);
		return menuBar;
	}
	
	
	/**
	 * Shows a panel with features list.
	 */
	private final Action myShowFeaturesAction = setupAction( new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
			new InfoFrame(Main.this, InfoFrame.FEATURES);
			
		}
	}, "Show features list");
	
	/**
	 * Shows hot keys list.
	 */
	private final Action myShowHotKeysAction = setupAction( new AbstractAction() {
		public void actionPerformed(ActionEvent e)
		{
			new InfoFrame(Main.this, InfoFrame.HOT_KEYS);
		}
	}, "Show hot keys list");
	
	/**
	 * Shows hot keys list.
	 */
	private final Action myShowAboutAction = setupAction( new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
			new InfoFrame(Main.this, InfoFrame.AUTHOR);
			
		}
	}, "About");
	
	/**
	 * Sets explorer style viewer as current.
	 */
	private final Action mySwitchToExplorerStyleAction = setupAction( new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
			if (myCurrentStyle != SupportedViewerStyles.EXPLORER_STYLE) {
				ViewerState state = myNCStyleViewer.getState();
				JPanel contentPane = (JPanel) getContentPane();
				contentPane.remove(myMainPanel);
				myMainPanel = myExplorerStyleViewer.getMainPanel();
				contentPane.add(myMainPanel);
				myExplorerStyleViewer.setState(state);
				contentPane.revalidate();
				contentPane.repaint();
				myCurrentStyle = SupportedViewerStyles.EXPLORER_STYLE;
				myViewerStyleButtonGroup.setSelected(mySwitchToExplorerStyleButton.getModel(), true);
			}
		}
	}, "Explorer style");
	
	/**
	 * Sets NC style viewer as current.
	 */
	private final Action mySwitchToNCStyleAction = setupAction( new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
			if (myCurrentStyle != SupportedViewerStyles.NC_STYLE) {
				ViewerState state = myExplorerStyleViewer.getState();
				JPanel contentPane = (JPanel) getContentPane();
				contentPane.remove(myMainPanel);
				myMainPanel = myNCStyleViewer.getMainPanel();
				contentPane.add(myMainPanel);
				myNCStyleViewer.setState(state);
				contentPane.revalidate();
				contentPane.repaint();
				myCurrentStyle = SupportedViewerStyles.NC_STYLE;
				myViewerStyleButtonGroup.setSelected(mySwitchToNCStyleButton.getModel(), true);
			}
		}
	}, "Norton commander style");
	
	private final Action mySwitchNextStyleAction = setupAction( new AbstractAction(){
		public void actionPerformed(ActionEvent e) {
			if (myCurrentStyle == SupportedViewerStyles.NC_STYLE) {
				mySwitchToExplorerStyleAction.actionPerformed(null);
			} else {
				mySwitchToNCStyleAction.actionPerformed(null);
			}
		}
	}, "Next style");
		
	/**
	 * Actions which are executed before exit.
	 */
	private final Action myExitAction = setupAction( new AbstractAction(){
		public void actionPerformed(ActionEvent e) {
			myLogger.info("User closed file manager's window.");
			saveState(STATE_FILE_NAME);
			System.exit(0);
		}
	}, "Exit");
	
	/**
	 * Sets action's settings
	 * @param action action
	 * @param name name to be displayed in menu
	 * @return action with settings
	 */
	private Action setupAction(Action action, String name) {
		action.putValue(Action.NAME, name);
		return action;
	}
	
	/**
	 * Creates Look & Feel menu
	 * @param frame main frame
	 * @return created menu.
	 */
	public JMenu createLookAndFeelMenu(final JFrame frame) {
		//this, 
        JMenu menu = new JMenu("Look & Feel");
        menu.setMnemonic(KeyEvent.VK_L);

        ButtonGroup group = new ButtonGroup();
        for (final UIManager.LookAndFeelInfo lafi : UIManager.getInstalledLookAndFeels()) {
            JRadioButtonMenuItem item = new JRadioButtonMenuItem(                
                new AbstractAction(lafi.getName()){
                    public void actionPerformed(ActionEvent event) {
                        try {
                            UIManager.setLookAndFeel(lafi.getClassName());
                            SwingUtilities.updateComponentTreeUI(frame);
                            //frame.pack();
                        } catch (Exception e) {
                        	myLogger.log(Level.SEVERE, "Cannot set look and feel!", e);
                        }
                    }
                }
            );
            group.add(item);
            menu.add(item);
        }
        group.setSelected(group.getElements().nextElement().getModel(), true);
        return menu;
    }
	
	private ComponentListener myResizeListener = new ComponentAdapter() {
		public void componentResized(ComponentEvent e) {
			Dimension dim = Main.this.getSize();
			int width = (dim.width < Settings.getMinFrameWidth() ? Settings.getMinFrameWidth() : dim.width);
			int height = (dim.height < Settings.getMinFrameHeight() ? Settings.getMinFrameHeight() : dim.height);
			Main.this.setSize(new Dimension(width, height));
		}
	};
	
	
	/**
	 * Checks if specified entity is this program's jar file
	 * @param entity entity to check
	 * @return result of the check 
	 */
	private boolean isMainJarFile(File file) {
		if (file.exists() && file.getName().endsWith("jar")) {
			try {
				JarFile jf = new JarFile(file);
				java.util.Enumeration<JarEntry> entries = jf.entries();
				while (entries.hasMoreElements()) {
					if (entries.nextElement().getName().indexOf("org/isakiev/fileManager") > -1) {
						return true;
					}
				}
			} catch (IOException e) {}
		}
		return false;
	}
	
	/**
	 * Tries to find in current directory program's jar file.  
	 */
	private void defineMyJarName() {
		String jarName = DEFAULT_JAR_NAME;
		File currentDir = new File(System.getProperty("user.dir"));
		for (File file:currentDir.listFiles()) {
			if (isMainJarFile(file)) {
				jarName = file.getAbsolutePath();
			}
		}
		Tools.getTools().setMainJarName(jarName);
	}
	
	/**
	 * Runs file manager.
	 * @param args command line arguments.
	 */
	public static void main(String[] args) {

		if (args.length > 0) {
			if (args[0].equals("-log")) {
				logging = true;
			}
		}
		
		try {
			new Main();
		} catch (RuntimeException e) { 
			Logger logger = Logger.getLogger("org.isakiev.fileManager");
			logger.log(Level.SEVERE, "Uncaught exception: " + e.getMessage(), e);
			throw e;
		} 
	}
	
	
	
}
