package org.isakiev.fileManager.archives;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.zip.*;
import java.util.logging.*;

import org.isakiev.fileManager.entities.*;
import org.isakiev.fileManager.entityTypes.EntityTypeContainer;

/**
 * This class provides operations with ZIP archives.
 * @author Ruslan Isakiev
 */
public class ZipArchiveManager implements IArchiveManager {
	
	private EntityComparator<IEntity> myComparator;
	private Logger myLogger;
	private static IArchiveManager myInstance = null;
	
	private ZipArchiveManager() {
		myComparator = new EntityComparator<IEntity>();
		myLogger = Logger.getLogger("org.isakiev.fileManager");
	}
	
	public static IArchiveManager getInstance() {
		if (myInstance == null) {
			myInstance = new ZipArchiveManager();
		}
		return myInstance;
	}

	public void exploreArchive(IDirEntity archive) {
		
		ArrayList<IEntity> empty = new ArrayList<IEntity>();
		archive.setChildren(empty);
		ZipFile zf = null;
		try {
	        zf = new ZipFile(archive.getFile());
	        //System.out.println(new PushBackEnum(zf.entries()));
	        PushBackEnum<ZipEntry> zipEntries = new PushBackEnum<ZipEntry>(zf.entries());
	        exploreZip(archive.getFile(), zipEntries, archive);

		} catch (ZipException ze) {
			myLogger.log(Level.SEVERE, "exploreDir >> ZipException (file \"" + archive.getFullName() + "\")", ze);
	    } catch (IOException ioe) {
	    	myLogger.log(Level.SEVERE, "exploreDir >> IOException (file \"" + archive.getFullName() + "\")", ioe);
	    } finally {
	    	try {
	    		if (zf != null) zf.close();
	    	} catch (IOException e) {}
	    }

	}
	
	/**
	 * Creates tree with zip archive content 
	 * @param archiveFile archive file name
	 * @param source zip file entries
	 * @param parent root node for this tree
	 */
	private void exploreZip(File archiveFile, PushBackEnum<ZipEntry> source, IDirEntity parent) {
		
		ArrayList<IEntity> children = new ArrayList<IEntity>();
	
		while (source.hasMoreEntries()) {
			ZipEntry zipEntry = source.nextEntry();
			
			boolean isDir = false;
			String name = zipEntry.getName();
			//System.out.println(zipEntry.getName());
			if (name.endsWith("/")) {
				isDir = true;
				name = name.substring(0, name.length() - 1);
			}
			
			int pos = name.lastIndexOf("/");
			
			if (pos == -1) {
				if (!parent.isArchiveEntity()) {
					source.pushBackEntry(zipEntry);
					//System.out.println("   pos=-1; parent is not archive");
					break;
				}
			} else {
				String prefix = name.substring(0,pos);
				//if path to this entry doesn't exist
				if (parent.isArchiveEntity()) {
					source.pushBackEntry(zipEntry);
					String[] prefixComponents = prefix.split("/");
					String[] prefixes = new String[prefixComponents.length];
					prefixes[0] = prefixComponents[0] + "/";
					for (int i=1; i<prefixes.length; i++) {
						prefixes[i] = prefixes[i-1] + prefixComponents[i] + "/";
					}
					for (int i=prefixes.length-1; i>=0; i--) {
						IEntity entity = EntityFactory.createCompressedDirEntity(archiveFile, zipEntry, parent, EntityTypeContainer.getInstance().getDirectoryType());

						source.pushBackEntry(new ZipEntry(prefixes[i]));
					}
					//System.out.println("   new prefixes = "+prefixes.length);
					continue;
				}
				String parentName = ((ICompressedEntity) parent).getZipEntry().getName();
				//System.out.println("   parent name = "+parentName);
				//System.out.println("   prefix = "+prefix);
				
				if (!parentName.equals(prefix + "/")) {
					source.pushBackEntry(zipEntry);
					//System.out.println("   parent name doesn't starts with prefix");
					break;
				}
			}
					
			//System.out.println("   adding entity");
			IEntity entity = null;
    		if (isDir) {
    			entity = EntityFactory.createCompressedDirEntity(archiveFile, zipEntry, parent, EntityTypeContainer.getInstance().getDirectoryType());
      			exploreZip(archiveFile, source, (IDirEntity) entity);

    		} else {
    			entity = EntityFactory.createCompressedFileEntity(archiveFile,zipEntry,parent,EntityTypeContainer.getInstance().getCompressedFileType());
    		}
    		
			children.add(entity);
		}
		Collections.sort(children, myComparator);
    	parent.setChildren(children);
	}

}
