package org.isakiev.fileManager.contentManagers;

import java.util.*;
import java.util.logging.*;
import java.io.*;

import org.isakiev.fileManager.entities.*;
import org.isakiev.fileManager.entityTypes.*;
import org.isakiev.fileManager.archives.*;



/**
 * Main content manager.
 * @author Ruslan Isakiev
 */
public class ContentManager implements IContentManager {
	
	private EntityTypeContainer myFileTypeContainer;
	private EntityComparator<IEntity> myComparator;
	private Logger myLogger;
	
	/**
	 * Main constructor.
	 */
	public ContentManager() {
		myFileTypeContainer = EntityTypeContainer.getInstance();
		myComparator = new EntityComparator<IEntity>();
		myLogger = Logger.getLogger("org.isakiev.fileManager");
	}
	
	/**
	 * Returns the root of file system tree with the children 
	 * (The children are the system root folders). 
	 * @return root of the file system tree
	 */
	public IDirEntity getRoot() {
		IDirEntity result = EntityFactory.createGlobalRootDirEntity();
		
		File[] fileSystemRoots = File.listRoots();
		ArrayList<IEntity> children = new ArrayList<IEntity>(fileSystemRoots.length);

		for (int i = 0; i < fileSystemRoots.length; i++) {
			children.add(i,EntityFactory.createDiskEntity(fileSystemRoots[i], result, myFileTypeContainer.getDiskType()));
		}

		result.setChildren(children);
		return result;

	}
	
	/**
	 * Explores directory and specifies the children - files and subdirectories.
	 * @param dir directory to explore.
	 */
	public void exploreDir(IDirEntity dir) {
		
		if (dir.isArchiveEntity()) {
			if (dir.getType().getName().equals("zip")) {
				ZipArchiveManager.getInstance().exploreArchive(dir);
			}
		} else if (!dir.isCompressedEntity()) {
			File[] files = dir.getFile().listFiles();
			if (files == null) {
				dir.setChildren(new ArrayList<IEntity>(0));
			} else {
				ArrayList<IEntity> children = new ArrayList<IEntity>(files.length);
				for (int i = 0; i < files.length; i++) {
					if (files[i].isDirectory()) {
						children.add(i,EntityFactory.createDirEntity(files[i], dir, myFileTypeContainer.getDirectoryType()));
					} else {
						IEntityType type = myFileTypeContainer.getFileType(files[i]);
						if (type.isArchive()) {
							children.add(i,EntityFactory.createArchiveEntity(files[i], dir, type));
						} else {
							children.add(i,EntityFactory.createFileEntity(files[i], dir, type));
						}
					}
				}
				Collections.sort(children,myComparator);
				dir.setChildren(children);
			}
			
		} 
			
	}

	/**
	 * Adds to specified filesList information about file and 
	 * subfiles if parameter is a directory.
	 * @param filesList files list
	 * @param file file or directory
	 */
	private static void appendFilesList(FilesList filesList, File file) {
		if (file.isDirectory()) {
			filesList.addDirectory(file);
			File[] children = file.listFiles();
			for (int i=0; i<children.length; i++) {
				appendFilesList(filesList, children[i]);
			}
		} else {
			filesList.addFile(file);
		}
	}
	
	/**
	 * Renames / moves entity to the specified directory
	 * @param entity entity to rename / to move
	 * @param newName new name for this entity
	 * @param listener listens to the process events
	 * @return true if operation completed successfully
	 */
	public boolean renMove(IEntity entity, String newName, IProcessInfoListener listener) {
		File sourceFile = entity.getFile();
		File destFile = new File(newName);

		if (!sourceFile.renameTo(destFile)) {
			listener.showMessage("Can't rename/move: \""+sourceFile.getAbsolutePath()+"\" "
								 +"to \""+newName+"\"!");
			return false;
		}
		return true;
	}
	
	/**
	 * Deletes the specified entity
	 * @param entity entity to delete
	 * @param listener listens to the process events
	 */
	public void delete(IEntity entity, IProcessInfoListener listener) {
		File file = entity.getFile();
		if (file.exists()) {
			listener.progressChanged(0);
			StringBuffer buffer = new StringBuffer();
			
			FilesList filesList = new FilesList();
			appendFilesList(filesList, file);
			
			long totalLength = (filesList.getTotalLength() != 0 ? filesList.getTotalLength() : 1);
			long processedLength = 0;
			//Delete files
			for (File f: filesList.getFiles()) {
				processedLength += f.length();
				listener.processingNewFile(f.getName());
				if (!f.delete()) {
					buffer.append("Can't delete \""+f.getAbsolutePath()+"\"");
				}
				listener.progressChanged(new Double(processedLength*100.0/totalLength).intValue());
			}
			
			//Delete directories
			listener.progressChanged(100);
			for (File f: filesList.getDirectories()) {
				if (!f.delete()) {
					buffer.append("Can't delete \""+f.getAbsolutePath()+"\"");
				}
			}
			String message = buffer.toString();
			if (message.length() > 0) {
				listener.showMessage(message);
			}
		} else {
			listener.showMessage("\""+file.getAbsolutePath()+"\" doesn't exist!");
		}
		
	}
	
	public boolean copyFile(File sourceFile, 
						     File destinationFile, 
						     long totalLength, 
						     long copiedLength, 
						     IProcessInfoListener listener) {
		
		InputStream is = null;
		OutputStream os = null;
 
		if (myLogger.isLoggable(Level.FINER)) {
			myLogger.finer("Copy file: \""+sourceFile.getAbsolutePath()+ "\" to \"" + destinationFile.getAbsolutePath()+"\".");
		}
		try {
			is = new FileInputStream(sourceFile);
			os = new FileOutputStream(destinationFile);

			byte[] b = new byte[1024*10];
		    int c = 0;
		    long copied = copiedLength;
		    
		    while ((c = is.read(b)) >= 0) {
		        os.write(b, 0, c);
		        copiedLength += c;
		        listener.progressChanged(new Double(copiedLength*100.0/totalLength).intValue());
		    }
		    if (myLogger.isLoggable(Level.FINER)) {
		    	myLogger.finer("copied length = " + (copiedLength - copied) + " from "+sourceFile.length());
		    }

		} catch (IOException e) {
			if (myLogger.isLoggable(Level.FINER)) {
				myLogger.finer("Can't copy - IOException: "+e.getMessage());
			}
			listener.showMessage("Can't copy \""+sourceFile.getAbsolutePath()+"\"\nIOException: " + e.getMessage());
			return false;
		} finally {
			if (is != null) {
				try {is.close(); }
				catch (IOException e) {}
			}
			if (os != null) {
				try {os.close(); }
				catch (IOException e) {}
			}
		}
		return true;
	}
	


	
	/**
	 * Copies entity. 
	 * @param entity entity to copy
	 * @param substring 
	 * @param replacement
	 * @param listener listens to the process events
	 */
	public void copy(IEntity entity, String substring, String replacement, IProcessInfoListener listener) {
		File file = entity.getFile();
		if (myLogger.isLoggable(Level.FINE)) {
			myLogger.fine("Copying entity. entity=["+entity.getFullName()+"]; substring=["+substring+"]; replacement=["+replacement+"].");
		}
		if (file.exists()) {
			FilesList filesList = new FilesList();
			appendFilesList(filesList, file);

			//Create directories
			for (File f: filesList.getDirectories()) {
				String srcName = f.getAbsolutePath();
				String destName = srcName.replace(substring, replacement);

				if (myLogger.isLoggable(Level.FINER)) {
					myLogger.finer("Creating directory: \"" + destName + "\"");
				}
				
				File dir = new File(destName);
				if (!dir.exists()) {
					if (!dir.mkdirs()) {
						myLogger.warning("Can't create directory: \""+destName+"\"!");
						listener.showMessage("Can't create directory: \""+destName+"\"!");
						return;
					}
				}
			}
			
			//Copy files
			listener.progressChanged(0);
			long totalLength = (filesList.getTotalLength() != 0 ? filesList.getTotalLength() : 1);
			long copiedLength = 0;
			
			for (File f:filesList.getFiles()) {
				String srcName = f.getAbsolutePath();
				String destName = srcName.replace(substring, replacement);
			
				File destFile = new File(destName); 
				listener.processingNewFile(f.getName());
				if (!copyFile(f,destFile,totalLength, copiedLength, listener)) {
					return; //The message was sent to the listener in the "copyFile"
				}
				copiedLength += f.length();
			}
		} else {
			myLogger.warning("["+file.getAbsolutePath()+"] doesn't exist!");
			listener.showMessage("\""+file.getAbsolutePath()+"\" doesn't exist!");
		}

	}
	
	/**
	 * Creates subdirectory with the specified name in the parent directory. 
	 * @param parentDirectory parent directory
	 * @param name new directory's name
	 * @return true if operation completed successfully
	 */
	public boolean createDirectory(IDirEntity parentDirectory, String name, IProcessInfoListener processInfoListener) {
		File dir = new File(parentDirectory.getFile(), name);
		if (!dir.mkdir()) {
			processInfoListener.showMessage("Directory \"" + dir.getAbsolutePath() + "\" wasn't created!");
			return false;
		}
		return true;
	}
	

}
