package org.isakiev.fileManager.contentModel;

import java.util.List;

import org.isakiev.fileManager.contentManagers.IProcessInfoListener;
import org.isakiev.fileManager.entities.*;

/**
 * Interface of the content model. Performs only read operations. 
 * @author Isakiev Ruslan
 */
public interface IContentModel {
	/**
	 * Returns the root of the file system tree.
	 * @return The tree root (its children are the root folders of file system). 
	 */
	public IDirEntity getRoot();
	
	/**
	 * Returns children of the specified parent.
	 * @param parent parent.
	 * @return children.
	 */
	public List<IEntity> getChildren(IDirEntity parent);
	
	/**
	 * Returns entity by entity path and creates all neccessary entities 
	 * on the way from the root (only if these entities are actually existing).
	 * @param path path to the entity
	 * @return entity or <code>null</code> if path does not exist
	 */
	public IEntity getEntity(String path);
	
	/**
	 * Adds model listener.
	 * @param listener listener
	 */
	public void addModelListener(IContentModelListener listener);
	
	/**
	 * Removes model listener.
	 * @param listener listener
	 */
	public void removeModelListener(IContentModelListener listener);
	
	/**
	 * Renames / moves entity to the specified directory
	 * @param entity entity to rename / to move
	 * @param newName new name for this entity
	 * @param listener listens to the process events
	 * @return true if operation completed successfully
	 */
	public boolean renMove(IEntity entity, String newName, IProcessInfoListener listener);
	
	/**
	 * Copies entity. 
	 * @param entity entity to copy
	 * @param substring 
	 * @param replacement
	 * @param listener listens to the process events
	 */
	public void copy(IEntity entity, String substring, String replacement, IProcessInfoListener listener);
	
	/**
	 * Deletes the specified entity
	 * @param entity entity to delete
	 * @param listener listens to the process events
	 */
	public void delete(IEntity entity, IProcessInfoListener listener);
	
	/**
	 * Creates subdirectory with the specified name in the parent directory. 
	 * @param parentDirectory parent directory
	 * @param name new directory's name
	 * @return true if operation completed successfully
	 */
	public boolean createDirectory(IDirEntity parentDirectory, String name, IProcessInfoListener processInfoListener);

	/**
	 * Checks if entity exists.
	 * @param entity entity to check
	 * @return result of the check
	 */
	public boolean entityExists(IEntity entity);
	
	/**
	 * Gets the nearest existing parent. 
	 * @param unexistingEntity unexisting entity
	 * @return the nearest existing parent
	 */
	public IEntity getEntityInsteadOfUnexisting(IEntity unexistingEntity);
	

}
