package org.isakiev.fileManager.contentViewers;

import org.isakiev.fileManager.entities.*;

/**
 * This class is used to synchronize selected entities when viewer styles are changing.
 * @author Isakiev Ruslan
 */
public class ViewerState {
	
	private SupportedViewerStyles myStyle;
	
	//for explorer style
	private IEntity myCurrentDirectory = null;
	private IEntity myCurrentEntity = null;
	
	//for nc style
	
	private IEntity myLeftDirectory = null;
	private IEntity myRightDirectory = null;
	
	private IEntity myLeftEntity = null;
	private IEntity myRightEntity = null;
	
	private int myActivePanelIndex = 0;
	private int myDividerLocation = -1;
	
	
	/**
	 * Default constructor.
	 * @param style current viewer's style.
	 */
	public ViewerState(SupportedViewerStyles style) {
		myStyle = style;
	}
	
	/**
	 * Returns current viewer's style.
	 * @return current viewer's style.
	 */
	public SupportedViewerStyles getStyle() {
		return myStyle;
	}
	
	/**
	 * Sets current state (for viewers with one panel).
	 * @param directory current directory
	 * @param entity current entity
	 */
	public void setExplorerState(IEntity directory, IEntity entity, int dividerLocation) {
		//for explorer style
		myCurrentDirectory = directory;
		myCurrentEntity = entity;
		myDividerLocation = dividerLocation;
		//for nc style
		if (myActivePanelIndex == 0) {
			myLeftDirectory = directory;
			myLeftEntity = entity;
		} else {
			myRightDirectory = directory;
			myRightEntity = entity;
		}
	}
	
	/**
	 * Sets current state (for viewers with two panels).
	 * @param leftDir current directory in the left panel 
	 * @param leftEntity current entity in the left panel
	 * @param rightDir current directory in the left panel 
	 * @param rightEntity current entity in the left panel
	 * @param activePanel active panel (0 - left; else - right)
	 */
	public void setNCState(IEntity leftDir, 
			               IEntity leftEntity, 
			               IEntity rightDir, 
			               IEntity rightEntity, 
			               int activePanel) {
		//for nc style
		myLeftDirectory = leftDir;
		myLeftEntity = leftEntity;
		myRightDirectory = rightDir;
		myRightEntity = rightEntity;
		myActivePanelIndex = (activePanel == 0 ? 0 : 1);

		//for explorer style
		myCurrentDirectory = (activePanel == 0 ? leftDir : rightDir);
		myCurrentEntity = (activePanel == 0 ? leftEntity : rightEntity);
	}
	
	/**
	 * Returns current entity (for the explorer style viewer).
	 * @return current entity
	 */
	public IEntity getCurrentEntity() {
		return myCurrentEntity;
	}
	
	/**
	 * Returns current directory (for the explorer style viewer).
	 * @return current directory
	 */
	public IEntity getCurrentDirectory() {
		return myCurrentDirectory;
	}
		
	/**
	 * Returns entity in the left panel (for the NC style viewer).
	 * @return entity in the left panel
	 */
	public IEntity getLeftEntity() {
		return myLeftEntity;
	}
	
	/**
	 * Returns current directory in the left panel (for the NC style viewer).
	 * @return current directory in the left panel
	 */
	public IEntity getLeftDirectory() {
		return myLeftDirectory;
	}
	
	/**
	 * Returns entity in the right panel (for the NC style viewer).
	 * @return entity in the right panel
	 */
	public IEntity getRightEntity() {
		return myRightEntity;
	}
	
	/**
	 * Returns current directory in the right panel (for the NC style viewer).
	 * @return current directory in the right panel
	 */
	public IEntity getRightDirectory() {
		return myRightDirectory;
	}
	
	/**
	 * Returns active panel index (for the NC style viewer).
	 * @return active panel index (0 or 1)
	 */
	public int getActivePanelIndex() {
		return myActivePanelIndex;
	}
	
	/**
	 * Returns split pane's divider location (for explorer style viewers).
	 * @return split pane's divider location
	 */
	public int getDividerLocation() {
		return myDividerLocation;
	}

	
}
