package org.isakiev.fileManager.contentViewers.explorer;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

import org.isakiev.fileManager.contentModel.*;
import org.isakiev.fileManager.contentViewers.IContentViewer;
import org.isakiev.fileManager.contentViewers.SupportedViewerStyles;
import org.isakiev.fileManager.contentViewers.ViewerState;
import org.isakiev.fileManager.entities.*;



/**
 * Content viewer with explorer style.
 * @author Isakiev Ruslan
 */
public class ExplorerStyleContentViewer implements IContentViewer  {
	
	private JPanel myPanelContainer;
	private JSplitPane myMainPanel;

	private IContentModel myContentModel;
	private JTree myTree;
	private ExplorerStyleTreeModel myTreeModel;
	
	private JScrollPane myTreePane;
	private JComponent myInfoPane;
	private static JScrollPane myEmptyPane = new JScrollPane();;

	
	
	/**
	 * Main constructor.
	 * @param content source content model
	 */
	public ExplorerStyleContentViewer(IContentModel content) {
		
		myContentModel = content;
		myTreeModel = new ExplorerStyleTreeModel(myContentModel);
		myTree = new JTree(myTreeModel);
		
		myTree.setCellRenderer(new ExplorerStyleTreeCellRenderer());
		myTree.addTreeSelectionListener(new MyTreeSelectionListener());
		
		myTreePane = new JScrollPane(myTree);
		myTreePane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		myTreePane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		myInfoPane = myEmptyPane;
		
		myMainPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, myTreePane, myInfoPane);
		myMainPanel.addComponentListener(new MyComponentListener());
		myMainPanel.setResizeWeight(0.5);
		myMainPanel.setDividerSize(8);
		myMainPanel.setOneTouchExpandable(true);
		
		myPanelContainer = new JPanel(new GridLayout(1,1));
		myPanelContainer.add(myMainPanel);

	}
	
	
	/**
	 * Returns viewer's current state
	 * @return current state
	 */
	public ViewerState getState() {
		ViewerState state = new ViewerState(SupportedViewerStyles.EXPLORER_STYLE);
		
		//get path to the selected node
		TreePath treePath = myTree.getSelectionPath();
		IEntity currentDirectory = null;
		IEntity currentEntity = null;
		if (treePath != null) {
			IEntity selectedEntity = (IEntity) treePath.getLastPathComponent();
			if (selectedEntity.isTreeRoot()) {
				currentDirectory = selectedEntity;
				currentEntity = null;
			} else if (selectedEntity.hasChildren()) {
				if (myTree.isExpanded(treePath)) {
					currentDirectory = selectedEntity;
					currentEntity = null;
				} else {
					currentDirectory = selectedEntity.getParent();
					currentEntity = selectedEntity;
				}
			} else {
				currentDirectory = selectedEntity.getParent();
				currentEntity = selectedEntity;
			}
		}

		state.setExplorerState(currentDirectory, currentEntity, myMainPanel.getDividerLocation());
		return state;
	}
	
	private TreePath getPathToTheEntity(IEntity entity) {
		if (entity == null) {
			return null;
		} else {

			//Create path from current entity to the root
			ArrayList<IEntity> pathList = new ArrayList<IEntity>();
			pathList.add(entity);
			int count = 1;
			while (entity.getParent() != null) {
				entity = entity.getParent();
				pathList.add(entity);
				count++;
			}
			//Reverse list and create array
			IEntity[] pathArray = new IEntity[count];
			int index = count - 1;
			for (IEntity e:pathList) {
				pathArray[index] = e;
				index--;
			}
			
			//Create tree path
			return new TreePath(pathArray);
		}
	}
	
	private void selectCurrentEntityInTheTree(IEntity entity) {
		TreePath treePath = getPathToTheEntity(entity);
		myTree.fireTreeExpanded(treePath);
		myTree.setSelectionPath(treePath);
		myTree.scrollPathToVisible(treePath);
	}

	/**
	 * Sets new viewer's current state
	 * @return viewer's state
	 */
	public void setState(ViewerState state) {
		
		TreePath dirPath = getPathToTheEntity(state.getCurrentDirectory());
		myTree.expandPath(dirPath);
		if (state.getCurrentEntity() == null) {
			myTree.setSelectionPath(dirPath);
			myTree.scrollPathToVisible(dirPath);
		} else {
			TreePath entityPath = getPathToTheEntity(state.getCurrentEntity());
			myTree.setSelectionPath(entityPath);
			myTree.scrollPathToVisible(entityPath);
		}
		
		//myNotResized = false;
		if (state.getDividerLocation() > -1) {
			myMainPanel.setDividerLocation(state.getDividerLocation());
		}
	}
	
	
	/**
	 * Sets current viewer component.
	 * @param viewer viewer component
	 */
	/*public void setViewerComponent(IEntityViewer viewer){
		JComponent component = viewer.getExplorerStyleViewer(null);
		replaceInfoPane(component);
	}*/
	
	/**
	 * Replaces current info pane with new one
	 * @param newPane new info pane
	 */
	private void replaceInfoPane(JComponent newPane) {
		int dividerLocation = myMainPanel.getDividerLocation();
		Dimension dim = myInfoPane.getSize();
		myMainPanel.remove(myInfoPane);
		myInfoPane = newPane;
		myInfoPane.setSize(dim);
		myMainPanel.setRightComponent(myInfoPane);
		myMainPanel.revalidate();
		myMainPanel.setDividerLocation(dividerLocation);
	}
	
	private static final BorderLayout LAYOUT = new BorderLayout();
	
	private class MyTreeSelectionListener implements TreeSelectionListener {
		
		public void valueChanged(TreeSelectionEvent event) {
			
			//get path to the selected node
			TreePath treePath = event.getNewLeadSelectionPath();
			
			//if there are no selected nodes
			if (treePath == null) {
				replaceInfoPane(myEmptyPane);
				return;
			}
			
			//get selected node 
			IEntity entity = (IEntity) treePath.getLastPathComponent();
			//if entity doesn't exist
			if (!myContentModel.entityExists(entity)) {
				JOptionPane.showMessageDialog(myMainPanel, "Entity doesn't exist!\n("+entity.getFullName()+")","File manager info",JOptionPane.INFORMATION_MESSAGE);
				entity = myContentModel.getEntityInsteadOfUnexisting(entity);
				myTreeModel.fireStructureChanged(getPathToTheEntity(entity));
				selectCurrentEntityInTheTree(entity);
			}
			
			//update main panel
			JPanel panel = new JPanel(LAYOUT);
			panel.add(new InfoLabel(entity),BorderLayout.PAGE_START);
			
			JComponent viewer = entity.getType().getViewer().getViewerComponent(entity); 
			panel.add(viewer);

			replaceInfoPane(new JScrollPane(panel));
			
		}

	}
	
	private class MyComponentListener extends ComponentAdapter {
		public void componentResized(ComponentEvent e) {
			int width = e.getComponent().getWidth() - myMainPanel.getDividerSize();
			int dl = new Long(Math.round(width*0.47)).intValue();
			myMainPanel.setDividerLocation(dl);
			myEmptyPane.setSize(width - dl, myMainPanel.getHeight());


		}
	}
	
	public JPanel getMainPanel() {
		return myPanelContainer;
	}
	
	/**
	 * Returns the style of this viewer
	 * @return viewer style
	 */
	public SupportedViewerStyles getStyle() {
		return SupportedViewerStyles.EXPLORER_STYLE;
	}
	


}
