package org.isakiev.fileManager.contentViewers.nc;

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.logging.*;

import javax.swing.*;

import org.isakiev.fileManager.contentModel.IContentModel;
import org.isakiev.fileManager.contentModel.IContentModelListener;
import org.isakiev.fileManager.entities.*;
import org.isakiev.fileManager.utils.Settings;
import org.isakiev.fileManager.utils.Tools;
import org.isakiev.fileManager.contentViewers.*;

/**
 * This class represents panel in NC style viewer.
 * @author Ruslan Isakiev
 *
 */
class NCPanel extends JComponent 
 	implements INCPanelModelListener, FocusListener{
	
	private IContentModel myContentModel;
	private NCPanelModel myPanelModel;
	private IPanelDispatcher myDispatcher;
	
	private Logger myLogger;
	
	//Coordinates used to display component
	private int myLeft;
	private int myRight;
	private int myCenter;
	private int myTop;
	private int myTextTop;
	private int myRectTop;
	private int myBottom;
	
	/**
	 * Creates NC panel.
	 * @param model content model
	 * @param dispatcher dispatcher to switch panels
	 */
	public NCPanel(IContentModel model, IPanelDispatcher dispatcher) {
		myLogger = Logger.getLogger("org.isakiev.fileManager");
		myContentModel = model;
		myContentModel.addModelListener(new ContentModelListener());
		myPanelModel = new NCPanelModel(myContentModel);
		myPanelModel.addPanelModelListener(this);
		
		myDispatcher = dispatcher;
		
		this.setFocusable(true);
		updateTitledBorder();
		
		addComponentListener(new PanelResizeListener());
		addKeyListener(new NCPanelKeyListener());
		addMouseListener(new NCPanelMouseListener());
		addFocusListener(this);
	}
	
	/**
	 * Updates panel's border according to information about current directory.
	 */
	private void updateTitledBorder() {
		int width = this.getWidth() - 20;
		IEntity currentDir = myPanelModel.getCurrentDirectoryEntity();
		String topTitle = (currentDir.isTreeRoot() ? "File system" : currentDir.getFullName());
		String bottomTitle = Tools.getFileSizeString(myPanelModel.getSizeOfFilesInCurrentDirectory()) 
							+ " in " + myPanelModel.getNumberOfFilesInCurrentDirectory() + " files";
		String space = " ";
		setBorder(NCStyleBorderFactory.createNCBorder(
				Settings.getInstance().getTextColor(),
				Settings.getInstance().getFont(),
				space + correctPathString(topTitle, width - 20) + space,
				space + correctString(bottomTitle, width - 20) + space));
	}
	
	/**
	 * Gets width of the string with the specified font.
	 * @param s string
	 * @param font font
	 * @return width
	 */
	private int getStringWidth(String s, Font font) {
		FontMetrics metrics = getFontMetrics(font);
		return metrics.stringWidth(s);
	}
	
	/**
	 * Cuts string to make its width equal to specified.
	 * @param s string to cut
	 * @param width maximum width
	 * @param font font
	 * @return string with width less or equal to the specified width. 
	 */
	private String fitString(String s, int width, Font font) {
		if (s.length() == 0) {
			return "";
		}
		
		FontMetrics metrics = getFontMetrics(Settings.getInstance().getFont());
		char[] chars = new char[s.length()];
		s.getChars(0,s.length(),chars,0);
		int count = 0;
		int totalWidth = metrics.charWidth(chars[0]);
		while (totalWidth < width) {
			count++;
			totalWidth += metrics.charWidth(chars[count]);
		}
		return s.substring(0,count);
	}
	

	private String correctString(String s, int width) {
		
		Font font = Settings.getInstance().getFont();
		if (getStringWidth(s,font) < width) {
			return s;
		} else {
			String mark = "...";
			int markWidth = getStringWidth(mark,font);
			return fitString(s, width - markWidth, font) + mark;
		}
	}
	
	private String correctPathString(String s, int width) {
		//Temporary
		return correctString(s, width);
	}
	
	
	private void checkInsets() {
		Insets insets = getInsets();
		myLeft = insets.left;
		myRight = getWidth() - insets.right;
		myCenter = myLeft + (myRight - myLeft) / 2;
		myTop = insets.top;
		myBottom = getHeight() - insets.bottom - 2;
		myTextTop = myTop + 13;
		myRectTop = myTextTop - Settings.getInstance().getNCPanelCellHeight() + 4;
	}
	
	private void drawCell(Graphics g, int row, int column, String info, Color color) {
		
		int height = Settings.getInstance().getNCPanelCellHeight();
		int width = myCenter - myLeft - 3;
		int x = (column == 0 ? myLeft : myCenter + 3);
		int y = myTextTop + row * height;
		
		if (myPanelModel.isSelected(row,column) && hasFocus()) {
			g.setColor(Settings.getInstance().getSelectionColor());
			g.fillRect(x, myRectTop + row * height, width, height);

		}
		g.setColor(color);
		g.drawString(correctString(info,width),x + 2,y);

	}
	
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		checkInsets();
		
		g.setColor(Settings.getInstance().getBackgroundColor());
		g.setFont(Settings.getInstance().getFont());
		g.fillRect(0,0,getWidth(), getHeight());
		g.setColor(Settings.getInstance().getTextColor());
		
		g.drawLine(myCenter,myTop,myCenter,myBottom);

		for (int column = 0; column < myPanelModel.getColumnsNumber(); column++) {
			for (int row = 0; row < myPanelModel.getRowsNumber(); row++) {
				
				if (myPanelModel.isUpperLevelMarker(row, column)) {
					drawCell(g,row,column,"..",Settings.getInstance().getTextColor());
					
				} else if (!myPanelModel.isEmpty(row,column)){
					IEntity entity = myPanelModel.getElementAt(row, column);
					Color color;
					if (entity.isDirEntity() || entity.isDiskEntity()) {
						color = Settings.getInstance().getDirectoryColor();
					} else if (entity.isArchiveEntity()){
						color = Settings.getInstance().getArchiveFileColor();
					} else if (entity.getType().isExecutable()){
						color = Settings.getInstance().getExecutableFileColor();
					} else {
						color = Settings.getInstance().getTextColor();
					}
					drawCell(g,row,column,entity.toString(),color);
				}
					
			}
		}

	}
	
	/**
	 * Gets current directory. 
	 * @return current element.
	 */
	public IDirEntity getCurrentDirectory() {
		return (IDirEntity) myPanelModel.getCurrentDirectoryEntity();
	}
	
	/**
	 * Gets selected element in the current directory. 
	 * If nothing is selected, <code>null</code> id returned.
	 */
	public IEntity getCurrentElement() {
		return myPanelModel.getSelectedEntity();
	}
	

	/**
	 * Sets current element. 
	 * @param element new entity which will be set as current element
	 */
	public void setCurrentElement(IDirEntity directory, IEntity element) {
		myPanelModel.setCurrentElement(directory, element);
	}
	
	/**
	 * Sets file system tree root as a current directory.
	 */
	public void setRootAsCurrentDirectory() {
		myPanelModel.setRootAsCurrentDirectory();
	}
	
	/**
	 * Runs executable entity
	 * @param executableEntity entity to run
	 */
	private void runExecutable(IEntity executableEntity) {
		try {
			Runtime.getRuntime().exec(executableEntity.getFile().getAbsolutePath());
		} catch (java.io.IOException ioe) {}
	}
	
	/**
	 * Called when user pressed enter.
	 */
	private void processEnterActions() {
		
		IEntity selectedEntity = myPanelModel.getSelectedEntity();
		if (selectedEntity == null) {
			selectedEntity = myPanelModel.getCurrentDirectoryEntity();
		}
		if (!myContentModel.entityExists(selectedEntity)) {
			String s = (selectedEntity.isFileEntity() ? "File" : "Directory");
			JOptionPane.showMessageDialog(NCPanel.this, s+" doesn't exist!\n("+selectedEntity.getFullName()+")","File manager info",JOptionPane.INFORMATION_MESSAGE);
			myPanelModel.updatePanelModel();
			return;
		}
		
		if (myPanelModel.canProcessEnterAction()) {
			myPanelModel.processEnterAction();
		} else {
			IEntity entity = myPanelModel.getSelectedEntity();
			if (entity == null) {
				myLogger.warning("current entity has to be not null!");
				return;
			}
			
			if (entity.isFileEntity()) {
				if (entity.getType().isExecutable()) {
					runExecutable(entity);
				} else {
					JComponent viewer = entity.getType().getViewer().getViewerComponent(entity); //   Factory().getNCStyleViewer(entity, myDispatcher);
					JComponent viewerPanel = new NCStyleEntityViewerPanel(entity, viewer, myDispatcher, true);
					myDispatcher.replaceComponent(this, viewerPanel);
				}
			}
		}
		
	}
	
	/**
	 * This class is used to change table's size after main panel's resize
	 */
	private class PanelResizeListener extends ComponentAdapter {
		public void componentResized(ComponentEvent e) {
			updateTitledBorder();
			checkInsets();
			int rowsNumber = (myBottom - myRectTop)/Settings.getInstance().getNCPanelCellHeight();
			myPanelModel.setRowsNumber(rowsNumber);
			myPanelModel.updatePanelModel();
		}
	}
	
	/**
	 * Implenmented from INCPanelModelListener interface.
	 */
	public void panelModelDataChanged(EventObject event) {
		updateTitledBorder();
		repaint();
	}
	
	public void focusGained(FocusEvent e) {
		repaint();
	}
 
	public void focusLost(FocusEvent e) {
		repaint();
	}
	
	/**
	 * Mouse actions listener.
	 */
	private class NCPanelMouseListener extends MouseAdapter {
		
		private int myCol;
		private int myRow;
		
		private boolean checkMousePosition(MouseEvent e) {

			int height = Settings.getInstance().getNCPanelCellHeight();
			int width = myCenter - myLeft - 3;
					
			for (int col = 0; col < myPanelModel.getColumnsNumber(); col++) {
				int x = (col == 0 ? myLeft : myCenter + 3);
				for (int row = 0; row < myPanelModel.getRowsNumber(); row++) {
					if (x < e.getX() && e.getX() < (x + width)
						&& (myRectTop + row * height) < e.getY() && e.getY() < (myRectTop + (row + 1) * height)) {
						myRow = row;
						myCol = col;
						return true; 
					}
				}
			}
			return false;
		}
		
		public void mouseClicked(MouseEvent e) {
			
			requestFocus();
			
			if (checkMousePosition(e)) {
				if (!myPanelModel.isEmpty(myRow, myCol) || myPanelModel.isUpperLevelMarker(myRow,myCol)) {
					myPanelModel.moveSelection(myRow, myCol);
					repaint();
					if (e.getClickCount() == 2) {
						processEnterActions();
					}
				}
			}
		}
	}
	
	/**
	 * Keyboard actions listener.
	 */
	private class NCPanelKeyListener extends KeyAdapter {
		
		public void keyPressed(KeyEvent e) {
			int kc = e.getKeyCode();
			if (kc == KeyEvent.VK_ENTER) {
				processEnterActions();
			} else if (kc == KeyEvent.VK_UP || kc == KeyEvent.VK_DOWN || kc == KeyEvent.VK_LEFT || kc == KeyEvent.VK_RIGHT) {
				myPanelModel.moveSelection(kc);
			}
		}
	}
	
	/**
	 * Updates panel information
	 */
	public void updatePanel() {
		myPanelModel.updatePanelModel();
		repaint();
	}
	
	private class ContentModelListener implements IContentModelListener {
		public void contentModelDataChanged() {
			updatePanel();
		}
		public void entityWasRenamed(IEntity entity, IEntity renamedEntity) {
			;
		}
	}
}
