package org.isakiev.fileManager.contentViewers.nc;

import javax.swing.*;

import java.awt.event.*;
import java.awt.*;
import java.io.File;
import java.util.List;

import org.isakiev.fileManager.contentModel.IContentModel;
import org.isakiev.fileManager.contentManagers.IProcessInfoListener;
import org.isakiev.fileManager.contentViewers.*;
import org.isakiev.fileManager.entities.*;

/**
 * Content viewer with norton commander style.
 * @author Isakiev Ruslan
 */
public class NCStyleContentViewer implements IContentViewer {
	
	private IContentModel myContentModel;
	private NCPanel[] myPanels;
	private JComponent[] myActiveComponents;
	private JPanel myPanelsOwner;
	private JPanel myMainPanel;
	private JComponent myCommandLine;
	
	private PanelDispatcher myDispatcher;
	private int myActivePanelIndex = 0;

	/**
	 * Main constructor.
	 * @param model file system model
	 */
	public NCStyleContentViewer(IContentModel model) {
		
		myDispatcher = new PanelDispatcher();
		myContentModel = model;
		
		myPanels = new NCPanel[2];
		myPanels[0] = new NCPanel(myContentModel, myDispatcher);
		myPanels[1] = new NCPanel(myContentModel, myDispatcher);
		
		FocusListener focusListener = new MyFocusListener();
		myPanels[0].addFocusListener(focusListener);
		myPanels[1].addFocusListener(focusListener);
		
		myPanels[1].requestFocus();
		myPanels[0].requestFocus();
		
		myActiveComponents = new JComponent[2];
		myActiveComponents[0] = myPanels[0];
		myActiveComponents[1] = myPanels[1];
		
		myPanelsOwner = new JPanel(new GridLayout(1,2));
		myPanelsOwner.add(myActiveComponents[0]); 
		myPanelsOwner.add(myActiveComponents[1]); 

		myMainPanel = new JPanel(new BorderLayout());
		myMainPanel.add(myPanelsOwner,BorderLayout.CENTER);
		myCommandLine = new CommandLine();
		myMainPanel.add(myCommandLine, BorderLayout.PAGE_END);
		
		InputMap inputMap = myMainPanel.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F1,KeyEvent.ALT_MASK), Actions.SelectRootDirectoryInTheLeftPanel);
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F2,KeyEvent.ALT_MASK), Actions.SelectRootDirectoryInTheRightPanel);
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_S,KeyEvent.ALT_MASK), Actions.SelectTheSameDirectory);
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F5,0), Actions.Copy);
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F6,0), Actions.Rename);
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F7,0), Actions.CreateDir);
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F8,0), Actions.Delete);
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,0), Actions.Delete);
		
		ActionMap actionMap = myMainPanel.getActionMap();
		actionMap.put(Actions.SelectRootDirectoryInTheLeftPanel, new SelectRootDirectoryInTheLeftPanel());
		actionMap.put(Actions.SelectRootDirectoryInTheRightPanel, new SelectRootDirectoryInTheRightPanel());
		actionMap.put(Actions.Copy, new CopyAction());
		actionMap.put(Actions.Rename, new RenameAction());
		actionMap.put(Actions.Delete, new DeleteAction());
		actionMap.put(Actions.CreateDir, new CreateDirAction());
		actionMap.put(Actions.SelectTheSameDirectory, new SelectTheSameDirectoryAction());
		
	}
	
	private enum Actions {
		SelectRootDirectoryInTheLeftPanel,
		SelectRootDirectoryInTheRightPanel,
		SelectTheSameDirectory,
		Copy, Rename, Delete, CreateDir,
	}
	
	private class SelectTheSameDirectoryAction extends AbstractAction {
		public void actionPerformed(ActionEvent e)  {
			IDirEntity directory = myPanels[myActivePanelIndex].getCurrentDirectory();
			IEntity element = myPanels[myActivePanelIndex].getCurrentElement();
			int i = (myActivePanelIndex + 1) % 2;
			myPanels[i].setCurrentElement(directory, element);
			myPanels[i].updatePanel();
		}
	}
	
	private boolean directoryHasEntityWithTheSameName(IDirEntity dir, String name) {
		List<IEntity> children = myContentModel.getChildren(dir);
		for (IEntity child:children) {
			if (child.getShortName().equals(name)) {
				return true;
			}
		}
		return false;
	}
	
	private class CreateDirAction extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			IDirEntity parentDir = myPanels[myActivePanelIndex].getCurrentDirectory();
			if (!myContentModel.entityExists(parentDir)) {
				JOptionPane.showMessageDialog(myMainPanel,"Directory \"" + parentDir.getFullName() + "\" doesn't exist!","Warning!",JOptionPane.WARNING_MESSAGE);
				myPanels[0].updatePanel();
				myPanels[1].updatePanel();
				return;
			}
			if (parentDir.isTreeRoot() || parentDir.isArchiveEntity() || parentDir.isCompressedEntity()) {
				JOptionPane.showMessageDialog(myMainPanel,"Can't create directory here!","Warning!",JOptionPane.WARNING_MESSAGE);
				return;
			}
			String result = JOptionPane.showInputDialog(myMainPanel, "Creating new directory in the \"" + parentDir.getFullName() +"\"\nEnter name:");
			if (result != null) {
				if (directoryHasEntityWithTheSameName(parentDir, result)) {
					JOptionPane.showMessageDialog(myMainPanel,"Directory with such name already exists!","Can't create directory!",JOptionPane.WARNING_MESSAGE);
					return;
				}
				if (myContentModel.createDirectory(parentDir, result, processInfoListener)) {
					myPanels[myActivePanelIndex].setCurrentElement(parentDir, myContentModel.getEntity(parentDir.getFullName() + File.separator + result));
				}
			}
		}
	}
	
	private class CopyAction extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			
			
			IEntity srcEntity = myPanels[myActivePanelIndex].getCurrentElement();
			IDirEntity destDir = myPanels[(myActivePanelIndex + 1) % 2].getCurrentDirectory();
			
			if (srcEntity == null) {
				return;
			}
			
			String entityType = (srcEntity.isDirEntity() ? "Directory" : "File");
			
			if (!myContentModel.entityExists(srcEntity)) {
				JOptionPane.showMessageDialog(myMainPanel,entityType + " \"" + srcEntity.getFullName() + "\" doesn't exist!","Can't copy!",JOptionPane.WARNING_MESSAGE);
				myPanels[0].updatePanel();
				myPanels[1].updatePanel();
				return;
			}
			if (srcEntity.isTreeRoot() || srcEntity.isDiskEntity() || srcEntity.isCompressedEntity()) {
				JOptionPane.showMessageDialog(myMainPanel,"Wrong source!","Can't copy!",JOptionPane.WARNING_MESSAGE);
				return;
			}
			if (destDir.isTreeRoot() || destDir.isArchiveEntity() || srcEntity.isCompressedEntity()) {
				JOptionPane.showMessageDialog(myMainPanel,"Wrong destination directory!","Can't copy!",JOptionPane.WARNING_MESSAGE);
				return;
			}
			if (!myContentModel.entityExists(destDir)) {
				JOptionPane.showMessageDialog(myMainPanel,"Directory \"" + destDir.getFullName() + "\" doesn't exist!","Can't copy!",JOptionPane.WARNING_MESSAGE);
				myPanels[0].updatePanel();
				myPanels[1].updatePanel();
				return;
			}
			if (myPanels[0].getCurrentDirectory().equals(myPanels[1].getCurrentDirectory())) {
				JOptionPane.showMessageDialog(myMainPanel,"Can't copy to the same directory!","Warning!",JOptionPane.WARNING_MESSAGE);
				return;
			}
			if (directoryHasEntityWithTheSameName(destDir, srcEntity.getShortName())) {
				JOptionPane.showMessageDialog(myMainPanel,"Destination directory has the " + entityType + " with the same name as source!","Can't copy!",JOptionPane.WARNING_MESSAGE);
				return;
			}
			
			int result = JOptionPane.showConfirmDialog(null, "Copy \""+ srcEntity.getFile().getAbsolutePath()+"\"\nto \""+destDir.getFile().getAbsolutePath(), "Copy?", JOptionPane.YES_NO_OPTION);
			if (result == JOptionPane.YES_OPTION) {
				String substr = srcEntity.getParent().getFile().getAbsolutePath();
				String substring = (substr.endsWith(File.separator) ? substr.substring(0,substr.length()-1) : substr);
				String replacement = destDir.getFile().getAbsolutePath();
				String message = "Copying \""+ srcEntity.getFile().getAbsolutePath()+"\"\nto \""+destDir.getFile().getAbsolutePath();
				//NCMessageWindow messageWindow = new NCMessageWindow(myMainPanel, message);
				myContentModel.copy(srcEntity, substring, replacement, processInfoListener);
				//messageWindow.dispose();
			}
		
		}
	}
	
	private IProcessInfoListener processInfoListener = new IProcessInfoListener() {
		
		public void progressChanged(int percents) {}
		public void processingNewFile(String name) {};
		
		public void showMessage(String message) {
			JOptionPane.showMessageDialog(null, message);
		}
	};
	
	private class DeleteAction extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			IEntity srcEntity = myPanels[myActivePanelIndex].getCurrentElement();
			if (srcEntity != null && !srcEntity.isDiskEntity()) {
				
				Object[] options = {"Yes", "No"};
				int result = JOptionPane.showOptionDialog(
						          myMainPanel, 
						          "Delete \""+ srcEntity.getFile().getAbsolutePath()+"\"?", 
						          "Are you sure?", 
						          JOptionPane.YES_NO_OPTION,
						          JOptionPane.QUESTION_MESSAGE,
						          null, 
						          options,
						          "No"); 
				
				if (result == JOptionPane.YES_OPTION) {
					myContentModel.delete(srcEntity, processInfoListener);
				}
			}
			
		}
	}
	
	
	private class RenameAction extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			IEntity srcEntity = myPanels[myActivePanelIndex].getCurrentElement();
			
			if (srcEntity == null) {
				return;
			}
			
			String entityType = (srcEntity.isDirEntity() ? "directory" : "file");
			
			if (!myContentModel.entityExists(srcEntity)) {
				JOptionPane.showMessageDialog(myMainPanel,entityType + " \"" + srcEntity.getFullName() + "\" doesn't exist!","Can't rename!",JOptionPane.WARNING_MESSAGE);
				myPanels[0].updatePanel();
				myPanels[1].updatePanel();
				return;
			}
			if (srcEntity.isTreeRoot() || srcEntity.isDiskEntity() || srcEntity.isCompressedEntity()) {
				JOptionPane.showMessageDialog(myMainPanel,"Wrong source!","Can't rename!",JOptionPane.WARNING_MESSAGE);
				return;
			}
			
			String result = JOptionPane.showInputDialog(myMainPanel, "Rename \"" + srcEntity.getFile().getName() +"\" to:",srcEntity.getFile().getAbsolutePath());
			if (result != null) {
				if (myContentModel.renMove(srcEntity,result, processInfoListener)) {
					IEntity resultEntity = myContentModel.getEntity(result);
					myPanels[myActivePanelIndex].setCurrentElement((IDirEntity) (resultEntity.getParent()), resultEntity);
				}
			}
		}
	}
	
	/**
	 * Marks focused panel as active.
	 */
	private class MyFocusListener implements FocusListener {
		public void focusGained(FocusEvent e) {
			myActivePanelIndex = (e.getSource() == myPanels[0] ? 0 : 1);
		}
		public void focusLost(FocusEvent e) {
		}
	}
	
	
	/**
	 * Alt-F1 action
	 */
	private class SelectRootDirectoryInTheLeftPanel extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			myPanels[0].setRootAsCurrentDirectory();
			myDispatcher.restoreDefaultComponent(0);
			myPanels[0].requestFocus();
			
		}
	}
	
	/**
	 * Alt-F2 action
	 */
	private class SelectRootDirectoryInTheRightPanel extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			myPanels[1].setRootAsCurrentDirectory();
			myDispatcher.restoreDefaultComponent(1);
			myPanels[1].requestFocus();
		}
	}
	
	/**
	 * Returns the style of this viewer
	 * @return viewer style
	 */
	public SupportedViewerStyles getStyle() {
		return SupportedViewerStyles.NC_STYLE;
	}
	
	/**
	 * Returns viewer panel.
	 */
	public JPanel getMainPanel() {
		return myMainPanel;
	}
	
	/**
	 * Returns viewer's current state
	 * @return current state
	 */
	public ViewerState getState() {

		ViewerState state = new ViewerState(SupportedViewerStyles.NC_STYLE);

		state.setNCState(myPanels[0].getCurrentDirectory(),
						 myPanels[0].getCurrentElement(), 
						 myPanels[1].getCurrentDirectory(),
				         myPanels[1].getCurrentElement(),
				         myActivePanelIndex);

		return state;
	}
	
	/**
	 * Sets new viewer's current state
	 * @return viewer's state
	 */
	public void setState(ViewerState state) {
		if (state.getStyle() == SupportedViewerStyles.NC_STYLE) {
			
			myPanels[0].setCurrentElement((IDirEntity) state.getLeftDirectory(), state.getLeftEntity());
			myPanels[1].setCurrentElement((IDirEntity)state.getRightDirectory(), state.getRightEntity());
			myActivePanelIndex = state.getActivePanelIndex();
			myPanels[myActivePanelIndex].requestFocus();
			
		} else {
			myPanels[myActivePanelIndex].setCurrentElement((IDirEntity) (state.getCurrentDirectory()), state.getCurrentEntity());
			myPanels[myActivePanelIndex].requestFocus();
		}
		
	}
	
	/**
	 * Panel Dispatcher is used to switch panels in the NC style viewer.
	 */
	private class PanelDispatcher implements IPanelDispatcher {
		
		/**
		 * Replaces component with specified index with a new component 
		 * @param componentIndex component index (0 - left, 1 - right)
		 * @param newComponent new component
		 */
		public void replaceComponent(int componentIndex,  JComponent newComponent) {
			myPanelsOwner.remove(myActiveComponents[0]);
			myPanelsOwner.remove(myActiveComponents[1]);
			myActiveComponents[componentIndex] = newComponent;
			myPanelsOwner.add(myActiveComponents[0]);
			myPanelsOwner.add(myActiveComponents[1]);
			
			myMainPanel.revalidate();
			myActiveComponents[componentIndex].requestFocus();	
			myActivePanelIndex = componentIndex; 
		}
		
		/**
		 * Restores default component with specified index 
		 * @param componentIndex component index (0 - left, 1 - right)
		 */
		public void restoreDefaultComponent(int componentIndex) {
			replaceComponent(componentIndex, myPanels[componentIndex]);
		}
		
		/**
		 * Returns index of the component
		 * @param component
		 * @return index of the component (-1 - unknown; 0 - left, 1 - right)
		 */
		private int getComponentIndex(JComponent component) {
			int componentIndex = -1;
			if (component == myActiveComponents[0]) {
				componentIndex = 0;
			} else if (component == myActiveComponents[1]) {
				componentIndex = 1;
			} 
			return componentIndex;
		}

		/**
		 * Replaces component with a new one
		 * @param sourceComponent component to be replaced
		 * @param newComponent new component
		 */
		public void replaceComponent(JComponent sourceComponent, JComponent newComponent) {
			int componentIndex = getComponentIndex(sourceComponent);
			if (componentIndex != -1) {
				replaceComponent(componentIndex, newComponent);
			}
		}
		
		/**
		 * Replaces component with a default one
		 * @param sourceComponent component to be replaced
		 */
		public void restoreDefaultComponent(JComponent sourceComponent) {
			int componentIndex = getComponentIndex(sourceComponent);
			if (componentIndex != -1) {
				restoreDefaultComponent(componentIndex);
			}
		}
		
	}
	



}
