package org.isakiev.fileManager.entityTypes;

import java.io.File;
import java.util.*;
import java.util.logging.*;


/**
 * This singleton contains all supported file types and helps to define 
 * the type of the <code>File</code>.
 * @author Isakiev Ruslan
 */
public class EntityTypeContainer {
	
	private static EntityTypeContainer myInstance = null;
	
	private ArrayList<AbstractType> myFileTypes;
	private AbstractType myDirType;
	private AbstractType myDiskType;
	private AbstractType myCompressedFileType;
	private Logger myLogger;
	
	
	/**
	 * Returns an instance of the FileTypeContainer.
	 * @return an instance of the FileTypeContainer
	 */
	public static EntityTypeContainer getInstance() {
		if (myInstance == null) {
			myInstance = new EntityTypeContainer();
		}
		return myInstance;
	}
	
	/**
	 * Main constructor. Loads all classes extending <code>FileType</code> 
	 * from <code>org.isakievRuslan.fileManager.fileTypes</code> package 
	 * using reflection.
	 */
	private EntityTypeContainer() {
		myLogger = Logger.getLogger("org.isakiev.fileManager");
		myFileTypes = new ArrayList<AbstractType>();
		
		myFileTypes.add(new TextType());
		myFileTypes.add(new UnknownType());
		
		myDirType = new DirectoryType();
		myDiskType = new DiskType();
		myCompressedFileType = new CompressedType();
		
		EntityTypeInfoLoader fileTypeLoader = EntityTypeInfoLoader.getInstance();
		fileTypeLoader.loadInfo("data/DefaultFileFormats.xml");
		myFileTypes.addAll(fileTypeLoader.getFileTypes());
		
		Collections.sort(myFileTypes, new EntityTypeComparator<AbstractType>());
		myLogger.fine("Loaded file types:\n" + toString());
		//System.out.println(toString());
		
	}
	
	/**
	 * Compares two entity types by priority
	 * @param <T> has to be <code>AbstractType</code> 
	 */
	private class EntityTypeComparator<T> implements Comparator<T>{
		
		public int compare(T arg1,T arg2) {

			Integer id1 = ((AbstractType)arg1).getPriority();
			Integer id2 = ((AbstractType)arg2).getPriority();

			return id1.compareTo(id2);
		}

	}
	
	/**
	 * Defines file type. 
	 * @param file file to define
	 * @return file type or <code>null</code> if 
	 * imposible to define it. 
	 */
	public IEntityType getFileType(File file) {
		FileInfo info = new FileInfo(file);
		for (AbstractType fileType:myFileTypes) {
			if (fileType.checkFile(info)) 
				return fileType;
		}
		return null;
	}
	
	/**
	 * Returns directory type
	 * @return directory type
	 */
	public IEntityType getDirectoryType() {
		return myDirType;
	}
	
	/**
	 * Returns disk type
	 * @return disk type
	 */
	public IEntityType getDiskType() {
		return myDiskType;
	}
	
	/**
	 * Returns compressed file type
	 * @return compressed file type
	 */
	public IEntityType getCompressedFileType() {
		return myCompressedFileType;
	}
	
	/**
	 * Shows a list of the supported file types
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		for (AbstractType ft:myFileTypes) {
			buffer.append(ft.getDescription()); 
			buffer.append(" (Priority: ").append(ft.getPriority()).append(")\n");
		}
		return buffer.toString();
	}
	

}
